@@ -39,7 +39,7 @@
 
 #define ICON_NAME_MIME_PREFIX           "gnome-mime-"
 
-#define SELF_THUMBNAIL_SIZE_THRESHOLD   16384
+#define SELF_THUMBNAIL_SIZE_THRESHOLD   32768
 
 
 /* Returns NULL for regular */
@@ -260,6 +260,7 @@ gnome_icon_lookup (GtkIconTheme               *icon_theme,
   char *mime_name;
   char *thumbnail;
   time_t mtime;
+  gboolean is_svg;
 
   /* For backwards compat we support GnomeIconTheme too */
   if (GNOME_IS_ICON_THEME (icon_theme))
@@ -285,11 +286,11 @@ gnome_icon_lookup (GtkIconTheme               *icon_theme,
 
   if (thumbnail_factory)
     {
+      is_svg = (strcmp (mime_type, "image/svg") == 0 ||
+		strcmp (mime_type, "image/svg+xml") == 0);
       if (flags & GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES &&
-	  (mimetype_supported_by_gdk_pixbuf (mime_type) ||
-	   ((strcmp (mime_type, "image/svg") == 0 ||
-	     strcmp (mime_type, "image/svg+xml") == 0) &&
-	    flags & GNOME_ICON_LOOKUP_FLAGS_ALLOW_SVG_AS_THEMSELVES))  &&
+	  ((!is_svg && mimetype_supported_by_gdk_pixbuf (mime_type)) ||
+	   ( is_svg && flags & GNOME_ICON_LOOKUP_FLAGS_ALLOW_SVG_AS_THEMSELVES))  &&
 	  strncmp (file_uri, "file:/", 6) == 0 &&
 	  file_info && file_info->size < SELF_THUMBNAIL_SIZE_THRESHOLD)
 	return gnome_vfs_get_local_path_from_uri (file_uri);
